@@ -43,9 +43,10 @@ package org.codehaus.plexus.compiler.javac;
 
 import org.codehaus.plexus.compiler.AbstractCompiler;
 import org.codehaus.plexus.compiler.CompilerConfiguration;
-import org.codehaus.plexus.compiler.CompilerError;
 import org.codehaus.plexus.compiler.CompilerException;
+import org.codehaus.plexus.compiler.CompilerMessage;
 import org.codehaus.plexus.compiler.CompilerOutputStyle;
+import org.codehaus.plexus.compiler.CompilerResult;
 import org.codehaus.plexus.util.FileUtils;
 import org.codehaus.plexus.util.Os;
 import org.codehaus.plexus.util.StringUtils;
@@ -67,7 +68,6 @@ import java.net.URL;
 import java.net.URLClassLoader;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Collections;
 import java.util.List;
 import java.util.Map;
 import java.util.NoSuchElementException;
@@ -80,7 +80,6 @@ import java.util.concurrent.CopyOnWriteArrayList;
  * @author <a href="mailto:matthew.pocock@ncl.ac.uk">Matthew Pocock</a>
  * @author <a href="mailto:joerg.wassmer@web.de">J&ouml;rg Wa&szlig;mer</a>
  * @author Others
- * @version $Id$
  * @plexus.component role="org.codehaus.plexus.compiler.Compiler"
  * role-hint="javac"
  */
@@ -115,7 +114,7 @@ public class JavacCompiler
     // Compiler Implementation
     // ----------------------------------------------------------------------
 
-    public List<CompilerError> compile( CompilerConfiguration config )
+    public CompilerResult performCompile( CompilerConfiguration config )
         throws CompilerException
     {
         File destinationDir = new File( config.getOutputLocation() );
@@ -129,7 +128,7 @@ public class JavacCompiler
 
         if ( ( sourceFiles == null ) || ( sourceFiles.length == 0 ) )
         {
-            return Collections.<CompilerError>emptyList();
+            return new CompilerResult();
         }
 
         if ( ( getLogger() != null ) && getLogger().isInfoEnabled() )
@@ -141,7 +140,7 @@ public class JavacCompiler
 
         String[] args = buildCompilerArguments( config, sourceFiles );
 
-        List<CompilerError> messages;
+        CompilerResult result;
 
         if ( config.isFork() )
         {
@@ -160,14 +159,36 @@ public class JavacCompiler
                 }
             }
 
-            messages = compileOutOfProcess( config, executable, args );
+            result = compileOutOfProcess( config, executable, args );
         }
         else
         {
-            messages = compileInProcess( args, config );
+            if ( isJava16() && !config.isForceJavacCompilerUse() )
+            {
+                // use fqcn to prevent loading of the class on 1.5 environment !
+                result =
+                    org.codehaus.plexus.compiler.javac.JavaxToolsCompiler.compileInProcess( args, config, sourceFiles );
+            }
+            else
+            {
+                result = compileInProcess( args, config );
+            }
         }
 
-        return messages;
+        return result;
+    }
+
+    protected static boolean isJava16()
+    {
+        try
+        {
+            Thread.currentThread().getContextClassLoader().loadClass( "javax.tools.ToolProvider" );
+            return true;
+        }
+        catch ( Exception e )
+        {
+            return false;
+        }
     }
 
     public String[] createCommandLine( CompilerConfiguration config )
@@ -211,8 +232,10 @@ public class JavacCompiler
 
             args.add( getPathString( sourceLocations ) );
         }
-
-        args.addAll( Arrays.asList( sourceFiles ) );
+        if ( !isJava16() || config.isForceJavacCompilerUse() || config.isFork() )
+        {
+            args.addAll( Arrays.asList( sourceFiles ) );
+        }
 
         if ( !isPreJava16( config ) )
         {
@@ -334,7 +357,7 @@ public class JavacCompiler
             args.add( value );
         }
 
-        return (String[]) args.toArray( new String[args.size()] );
+        return args.toArray( new String[args.size()] );
     }
 
     /**
@@ -403,10 +426,10 @@ public class JavacCompiler
      * @param config     compiler configuration
      * @param executable name of the executable to launch
      * @param args       arguments for the executable launched
-     * @return List of CompilerError objects with the errors encountered.
+     * @return a CompilerResult object encapsulating the result of the compilation and any compiler messages
      * @throws CompilerException
      */
-    List<CompilerError> compileOutOfProcess( CompilerConfiguration config, String executable, String[] args )
+    protected CompilerResult compileOutOfProcess( CompilerConfiguration config, String executable, String[] args )
         throws CompilerException
     {
         Commandline cli = new Commandline();
@@ -450,7 +473,7 @@ public class JavacCompiler
 
         int returnCode;
 
-        List<CompilerError> messages;
+        List<CompilerMessage> messages;
 
         if ( ( getLogger() != null ) && getLogger().isDebugEnabled() )
         {
@@ -489,41 +512,30 @@ public class JavacCompiler
             throw new CompilerException( "Error while executing the external compiler.", e );
         }
 
-        if ( ( returnCode != 0 ) && messages.isEmpty() )
-        {
-            if ( err.getOutput().length() == 0 )
-            {
-                throw new CompilerException(
-                    "Unknown error trying to execute the external compiler: " + EOL + cli.toString() );
-            }
-            else
-            {
-                messages.add( new CompilerError(
-                    "Failure executing javac,  but could not parse the error:" + EOL + err.getOutput(), true ) );
-            }
-        }
-
-        return messages;
+        boolean success = returnCode == 0;
+        return new CompilerResult( success, messages );
     }
 
     /**
      * Compile the java sources in the current JVM, without calling an external executable,
      * using <code>com.sun.tools.javac.Main</code> class
      *
-     * @param args arguments for the compiler as they would be used in the command line javac
-     * @return List of CompilerError objects with the errors encountered.
+     * @param args   arguments for the compiler as they would be used in the command line javac
+     * @param config compiler configuration
+     * @return a CompilerResult object encapsulating the result of the compilation and any compiler messages
      * @throws CompilerException
      */
-    List<CompilerError> compileInProcess( String[] args, CompilerConfiguration config )
+    CompilerResult compileInProcess( String[] args, CompilerConfiguration config )
         throws CompilerException
     {
         final Class<?> javacClass = getJavacClass( config );
         final Thread thread = Thread.currentThread();
         final ClassLoader contextClassLoader = thread.getContextClassLoader();
         thread.setContextClassLoader( javacClass.getClassLoader() );
+        getLogger().debug( "ttcl changed run compileInProcessWithProperClassloader" );
         try
         {
-            return compileInProcess0( javacClass, args );
+            return compileInProcessWithProperClassloader(javacClass, args);
         }
         finally
         {
@@ -532,17 +544,22 @@ public class JavacCompiler
         }
     }
 
+    protected CompilerResult compileInProcessWithProperClassloader( Class<?> javacClass, String[] args )
+        throws CompilerException {
+      return compileInProcess0(javacClass, args);
+    }
+
     /**
      * Helper method for compileInProcess()
      */
-    private static List<CompilerError> compileInProcess0( Class<?> javacClass, String[] args )
+    private static CompilerResult compileInProcess0( Class<?> javacClass, String[] args )
         throws CompilerException
     {
         StringWriter out = new StringWriter();
 
         Integer ok;
 
-        List<CompilerError> messages;
+        List<CompilerMessage> messages;
 
         try
         {
@@ -569,28 +586,22 @@ public class JavacCompiler
             throw new CompilerException( "Error while executing the compiler.", e );
         }
 
-        if ( ( ok.intValue() != 0 ) && messages.isEmpty() )
-        {
-            // TODO: exception?
-            messages.add( new CompilerError( "Failure executing javac, but could not parse the error:" + EOL +
-                                                 out.toString(), true ) );
-        }
-
-        return messages;
+        boolean success = ok.intValue() == 0;
+        return new CompilerResult( success, messages );
     }
 
     /**
-     * Parse the output from the compiler into a list of CompilerError objects
+     * Parse the output from the compiler into a list of CompilerMessage objects
      *
      * @param exitCode The exit code of javac.
      * @param input    The output of the compiler
-     * @return List of CompilerError objects
+     * @return List of CompilerMessage objects
      * @throws IOException
      */
-    static List<CompilerError> parseModernStream( int exitCode, BufferedReader input )
+    static List<CompilerMessage> parseModernStream( int exitCode, BufferedReader input )
         throws IOException
     {
-        List<CompilerError> errors = new ArrayList<CompilerError>();
+        List<CompilerMessage> errors = new ArrayList<CompilerMessage>();
 
         String line;
 
@@ -608,13 +619,18 @@ public class JavacCompiler
 
                 if ( line == null )
                 {
+                    // javac output not detected by other parsing
+                    if ( buffer.length() > 0 && buffer.toString().startsWith( "javac:" ) )
+                    {
+                        errors.add( new CompilerMessage( buffer.toString(), CompilerMessage.Kind.ERROR ) );
+                    }
                     return errors;
                 }
 
                 // TODO: there should be a better way to parse these
                 if ( ( buffer.length() == 0 ) && line.startsWith( "error: " ) )
                 {
-                    errors.add( new CompilerError( line, true ) );
+                    errors.add( new CompilerMessage( line, true ) );
                 }
                 else if ( ( buffer.length() == 0 ) && isNote( line ) )
                 {
@@ -647,13 +663,13 @@ public class JavacCompiler
     }
 
     /**
-     * Construct a CompilerError object from a line of the compiler output
+     * Construct a CompilerMessage object from a line of the compiler output
      *
      * @param exitCode The exit code from javac.
      * @param error    output line from the compiler
-     * @return the CompilerError object
+     * @return the CompilerMessage object
      */
-    static CompilerError parseModernError( int exitCode, String error )
+    static CompilerMessage parseModernError( int exitCode, String error )
     {
         StringTokenizer tokens = new StringTokenizer( error, ":" );
 
@@ -710,7 +726,7 @@ public class JavacCompiler
 
             if ( startOfFileName > -1 )
             {
-                file = file.substring( startOfFileName + 2 );
+                file = file.substring( startOfFileName + 1 + EOL.length() );
             }
 
             int line = Integer.parseInt( lineIndicator );
@@ -775,19 +791,19 @@ public class JavacCompiler
                 endcolumn = context.length();
             }
 
-            return new CompilerError( file, isError, line, startcolumn, line, endcolumn, message.trim() );
+            return new CompilerMessage( file, isError, line, startcolumn, line, endcolumn, message.trim() );
         }
         catch ( NoSuchElementException e )
         {
-            return new CompilerError( "no more tokens - could not parse error message: " + error, isError );
+            return new CompilerMessage( "no more tokens - could not parse error message: " + error, isError );
         }
         catch ( NumberFormatException e )
         {
-            return new CompilerError( "could not parse error message: " + error, isError );
+            return new CompilerMessage( "could not parse error message: " + error, isError );
         }
         catch ( Exception e )
         {
-            return new CompilerError( "could not parse error message: " + error, isError );
+            return new CompilerMessage( "could not parse error message: " + error, isError );
         }
     }
 
@@ -969,7 +985,7 @@ public class JavacCompiler
     /**
      * Helper method for create Javac class
      */
-    private Class<?> createJavacClass()
+    protected Class<?> createJavacClass()
         throws CompilerException
     {
         try
@@ -991,8 +1007,13 @@ public class JavacCompiler
 
         try
         {
-            final ClassLoader javacClassLoader =
-                new URLClassLoader( new URL[]{ toolsJar.toURI().toURL() }, JavacCompiler.class.getClassLoader() );
+            // Combined classloader with no parent/child relationship, so classes in our classloader
+            // can reference classes in tools.jar
+            URL[] originalUrls = ((URLClassLoader) JavacCompiler.class.getClassLoader()).getURLs();
+            URL[] urls = new URL[originalUrls.length + 1];
+            urls[0] = toolsJar.toURI().toURL();
+            System.arraycopy(originalUrls, 0, urls, 1, originalUrls.length);
+            ClassLoader javacClassLoader = new URLClassLoader(urls);
 
             final Thread thread = Thread.currentThread();
             final ClassLoader contextClassLoader = thread.getContextClassLoader();
